// This is the part of MPFRCPP, the C++ interface for MPFR library.

// Copyright (C) 2006 Alexey Beshenov <bav.272304@gmail.com>

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
// MA 02110-1301 USA.

#ifndef __FITS_CPP
#define __FITS_CPP

//------------------------------------------------------------
// MPFRCPP FITS()
//------------------------------------------------------------

// This is the special code for MPFRCPP based on
// ISO/IEC 14882:1998, section 18.2.1, `Numeric limits'.

// numeric_limits<T> is used for
// T = short, int, long, unsigned short, unsigned int,
// unsigned long, float, double, long double.

// For types long int and unsigned long int call
// fits<int>() and fits<unsigned int>().

template<typename T> bool MPFR::fits () const
    {

    if (is_number(*this)) {

        if (*this < MPFR(0) && std::numeric_limits<T>::is_signed == false)
            return false;

        if (*this < MPFR(std::numeric_limits<T>::min()))
            return false;

        if (*this > MPFR(std::numeric_limits<T>::max()))
            return false;

        if (std::numeric_limits<T>::is_integer == false) {
            if (this->exp() < std::numeric_limits<T>::min_exponent)
                return false;
            if (this->exp() > std::numeric_limits<T>::max_exponent)
                return false;
            }

        }

    else {

        if (is_nan(*this) && (std::numeric_limits<T>::has_quiet_NaN == false || std::numeric_limits<T>::has_signaling_NaN == false))
            return false;

        if (is_inf(*this) && std::numeric_limits<T>::has_infinity == false)
            return false;

        }

    return true;

    }

#endif                                            // __FITS_CPP
